# **********************************************************************
# Copyright 2017 VMware, Inc.  All rights reserved. VMware Confidential.
# **********************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
#
# **********************************************************************

import nsx_logging
import re
import json

import cli.commands.host_shared.ovs_util as ovs_util

from cli.commands.cli_command_provider import CliCommandProvider

lg = nsx_logging.getLogger("cli.commands.hostpubliccloud.firewall")


class GetDfwRuleSummary(CliCommandProvider):
    def __call__(self, cmd_list):
        res = []
        vifs = ovs_util.get_firewall_interfaces()
        for vif in vifs:
            res.append(dict())
            vif_id = vif.get('vifuuid')
            res[-1]['vifuuid'] = vif_id
            res[-1]['rulesets'] = ovs_util.get_firewall_rules_vif(vif_id)

        return "OK\n\n%s" % json.dumps(res)


class GetDfwAddrsetSummary(CliCommandProvider):
    def __call__(self, cmd_list):
        res = []
        vifs = ovs_util.get_firewall_interfaces()
        for vif in vifs:
            res.append(dict())
            res[-1]['vifuuid'] = vif.get('vifuuid')
            res[-1]['addrsets'] = []
            addrs = ovs_util.get_firewall_address_sets_vif(res[-1]['vifuuid'])
            for addr in addrs:
                s = addr.split()
                if not s or s[-1] == '}':
                    continue
                elif s[-1] == '{':
                    res[-1]['addrsets'].append(dict())
                    res[-1]['addrsets'][-1]['uuid'] = s[0]
                    res[-1]['addrsets'][-1]['addrs'] = []
                elif s[-1][-1] == ',':
                    res[-1]['addrsets'][-1]['addrs'].append(s[-1][:-1])

        return "OK\n\n%s" % json.dumps(res)


class GetDfwAddrsetsName(CliCommandProvider):
    def __call__(self, cmd_list):
        res = []
        addrs = ovs_util.get_firewall_address_set_name(cmd_list[-1])
        # each line of address ends with comma "," #
        for addr in addrs:
            res.append(addr.split(',')[0])
        return "OK\n\n%s" % json.dumps(res)


class GetDfwExcludeVifs(CliCommandProvider):
    def __call__(self, cmd_list):
        res = []
        # each line for vif ends with comma "," #
        vifs = ovs_util.get_firewall_exclude_vifs()
        for vif in vifs:
            res.append(vif.split(',')[0])
        return "OK\n\n%s" % json.dumps(res)


class FirewallShowContextProfiles(CliCommandProvider):
    def __call__(self, cmd_list):
        res = []
        output = ovs_util.get_firewall_context_profiles()
        for line in output:
            vals = line.split(":", 1)
            if len(vals) == 2:
                key = vals[0].strip()
                value = vals[1].strip()
                if key == "Profile ID":
                    res.append(dict())
                    res[-1]['id'] = value
                    res[-1]['appid'] = []
                    res[-1]['fqdn'] = []

                if key == 'APP_ID':
                    res[-1]['appid'].append(value)

                if key == 'FQDN':
                    res[-1]['fqdn'].append(value.replace('\\', ''))

        return "OK\n\n%s" % json.dumps(res)


class FirewallShowFqdn(CliCommandProvider):
    def __call__(self, cmd_list):
        context_profile_id = cmd_list[3]
        cmd_list = [context_profile_id]
        output = ovs_util.get_firewall_context_profile_fqdn(cmd_list)
        res = []
        for line in output:
            vals = line.split(":", 1)
            if len(vals) == 2:
               key = vals[0].strip()
               value = vals[1].strip()
               if key == "FQDNs for Profile ID":
                   res.append(dict())
                   res[-1]['id'] = value
                   res[-1]['fqdnid'] = []

               if key == "FQDN ID":
                   res[-1]['fqdnid'].append(dict())
                   res[-1]['fqdnid'][-1]['id'] = value

               if key == "FQDN":
                   res[-1]['fqdnid'][-1]['url'] = value.replace('\\', '')

               if key == "Resolved IP":
                   res[-1]['fqdnid'][-1]['ip'] = []
                   if len(value) > 1:
                       ips = [x.strip() for x in value.split(',')]
                       for ip in ips:
                           res[-1]['fqdnid'][-1]['ip'].append(ip)

        return "OK\n\n%s" % json.dumps(res)
