#!/usr/bin/env python

import argparse
import logging
import os
import sys
import time

# Workaround for rhel environment, where the default
# python-protobuf is 2.5, which is incompatible with
# the built python proto bindings.
# This forces python to load the python proto bindings
# from nsx-python-protobuf.
if os.path.isfile('/etc/redhat-release'):
    if ('google' in sys.modules and
        sys.modules['google'].__path__ !=
            '/opt/vmware/nsx-common/lib/python/google'):
        del sys.modules['google']
    import google

import client_engine

# Interface imports:
from interfaces import proxy_config
from interfaces import proxy_metrics
from interfaces import rpc_info

# Interface instances:
interfaces = [proxy_config.ProxyConfig(), proxy_metrics.Metrics(), rpc_info.RpcInfo()]

def main():
    def checkInput(input):
        try:
            if (len(input) != 2):
                raise Exception('input format should be interfaceName:command')
            elif input[0] not in cli.subtools.keys():
                raise Exception('interfaceName %s does not exist' %(input[0]))
            elif len(input[1]) == 0:
                raise Exception('command cannot be empty')
            else:
                return True
        except Exception as e:
            print('Invalid Input: %s' % (e))
            return False

    parser = argparse.ArgumentParser(
        description='nsx-proxy CLI client',
        formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument(
        '-c', '--cmd', nargs='+',
        help='Run a single CLI command, \ninput format: interfaceName:command')
    parser.add_argument(
        '-e', '--endpoint', nargs='+',
        help=argparse.SUPPRESS)
    # pass in the path of log file you want, it has to be a valid directory path
    # with a filename, new logs will be appended to the file if it already exists
    parser.add_argument(
        '-l', '--logpath', nargs='+',
        help=argparse.SUPPRESS
    )

    args = parser.parse_args()

    if not args.endpoint:
        print('Missing input: endpoint')
        sys.exit(os.EX_NOINPUT)

    # config logger
    if args.logpath:
        log_path = args.logpath[0]
        # try create log file
        if not os.path.exists(log_path):
            with open(log_path, 'w'): pass

        logging.basicConfig(filename=log_path,
                            format='%(asctime)s %(message)s',
                            datefmt='%m/%d/%Y %I:%M:%S %p',
                            level=logging.DEBUG)
        logging.Formatter.converter = time.gmtime
        logger = logging.getLogger("nsxrpc")

    cli = client_engine.CliTool(interfaces, args.endpoint[0])
    if args.cmd:
        input = args.cmd[0].split(':')
        if (checkInput(input)):
            interfaceName, command = input[0], input[1]
            cli.subtools[interfaceName].onecmd(command)
    else:
        cli.cmdloop()

if __name__ == "__main__":
    main()
